%*********************************************************************
%  *
%  * Software License Agreement (BSD License)
%  *
%  *  Copyright (c) 2012, Scott Niekum
%  *  All rights reserved.
%  *
%  *  Redistribution and use in source and binary forms, with or without
%  *  modification, are permitted provided that the following conditions
%  *  are met:
%  *
%  *   * Redistributions of source code must retain the above copyright
%  *     notice, this list of conditions and the following disclaimer.
%  *   * Redistributions in binary form must reproduce the above
%  *     copyright notice, this list of conditions and the following
%  *     disclaimer in the documentation and/or other materials provided
%  *     with the distribution.
%  *   * Neither the name of the Willow Garage nor the names of its
%  *     contributors may be used to endorse or promote products derived
%  *     from this software without specific prior written permission.
%  *
%  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
%  *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
%  *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
%  *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
%  *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
%  *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
%  *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
%  *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
%  *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
%  *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
%  *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
%  *  POSSIBILITY OF SUCH DAMAGE.
%  *
%  *********************************************************************/

function [totData] = loadRobotLogs(whicharm)
    cellData = {};
    totData = struct;
    savelist = {};
    numlist = []
    
    basename = '/directoryNameWithFilesInThem';
    cd(basename);
    folders = dir;
    
    for i=1:size(folders)
        if(folders(i).isdir == 1 && ~strcmp(folders(i).name,'..') && ~strcmp(folders(i).name,'.'))
            foldname = strcat(basename,'/',folders(i).name);
            cd(foldname); 
            matlist = dir('Mat*');
           
            for j=1:size(matlist)
                matname = strcat(foldname,'/',matlist(j).name);
                s_ind = regexp(matlist(j).name, '[0-9]')
                s_num = matlist(j).name(s_ind)
                n = str2num(s_num)
                
                %Only add this data if it belongs to the correct arm
                %Even=right Odd=left
                if(mod(n,2) == whicharm)
                    d = load(matname);
                    cellData = [cellData; {d}];
                    s = strcat(foldname,'/Parse',s_num,'.txt');
                    savelist = [savelist, {s}]
                    numlist = [numlist,n]
                end
            end
            cd(basename);
        end
    end
    
    for i=1:size(cellData)
        [n,dims] = size(cellData{i}); 
        labels = ones(1,n);
        totData(i).obs = cellData{i}';
        totData(i).true_labels = labels;
        totData(i).savepath = savelist{i};
    end
end


  
    